"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.formatter = exports.po_box = exports.undocumentedRule = exports.rule7 = exports.rule6 = exports.rule5 = exports.rule4 = exports.rule3 = exports.rule2 = exports.rule1 = void 0;
const utils_1 = require("./utils");
/**
 * Rule 1 - No building name, number or sub building name
 * No premise elements detected (typically organisation name)
 */
const rule1 = (address) => {
    return utils_1.combinePremise(utils_1.premiseLocalities(address), address, "", "", "");
};
exports.rule1 = rule1;
/**
 * Rule 2 - Building number only
 */
const rule2 = (address) => {
    const { building_number } = address;
    const result = utils_1.premiseLocalities(address);
    utils_1.prependLocality(result, building_number);
    return utils_1.combinePremise(result, address, building_number, building_number, "");
};
exports.rule2 = rule2;
/**
 * Rule 3 - Building name only
 *
 * Check format of Building Name (see note (a) above). If the Exception
 * Rule applies, the Building Name should appear at the beginning of the
 * first Thoroughfare line, or the first Locality line if there is no
 * Thoroughfare information.
 *
 * When a building has a name AND a number range, both must be held in the
 * Building Name field because the Building Number field can only hold numeric
 * characters.
 *
 * If an address has a building name with text followed by a space and then
 * completed by numerics/numeric ranges with the numeric part an exception
 * (see Note (a) above), the numerics/numeric range are treated as a building
 * number, and the text part is treated as the Building Name and the
 * numerics/numeric range are split off to appear at the beginning of the first
 * Thoroughfare line, or the first Locality line if there is no Thoroughfare.
 */
const rule3 = (address) => {
    const { building_name } = address;
    let premise;
    let number = "";
    const result = utils_1.premiseLocalities(address);
    if (utils_1.nameException(building_name)) {
        premise = building_name;
        number = building_name;
        utils_1.prependLocality(result, utils_1.formatElem(premise));
    }
    else {
        const sub_range_match = utils_1.checkBuildingRange(building_name);
        if (sub_range_match && !utils_1.hasUnitPrefix(building_name)) {
            // Check if name contains number range
            premise = `${sub_range_match.name}, ${sub_range_match.range}`;
            number = sub_range_match.range;
            utils_1.prependLocality(result, sub_range_match.range);
            result.push(sub_range_match.name);
        }
        else {
            premise = building_name;
            result.push(premise);
        }
    }
    return utils_1.combinePremise(result, address, premise, number, "");
};
exports.rule3 = rule3;
// Building Name and Building Number
/**
 * Rule 4 - Building Name and Number
 *
 * The Building Name should appear on the line preceding the Thoroughfare
 * and/or Locality information. The Building Number should appear at the
 * beginning of the first Thoroughfare line. If there is no Thoroughfare
 * information then the Building Number should appear at the beginning of
 * the first Locality line.
 */
const rule4 = (address) => {
    const { building_name, building_number } = address;
    const result = utils_1.premiseLocalities(address);
    utils_1.prependLocality(result, building_number);
    result.push(building_name);
    return utils_1.combinePremise(result, address, `${building_name}, ${building_number}`, building_number, "");
};
exports.rule4 = rule4;
const STARTS_CHAR_REGEX = /^[a-z]$/i;
/**
 * Rule 5 - Sub Building Name and Building Number
 *
 * The Sub Building Name should appear on the line preceding the Thoroughfare
 * and Locality information. The Building Number should appear at the beginning
 * of the first Thoroughfare line. If there is no Thoroughfare information then
 * the Building Number should appear at the beginning of the first Locality line.
 */
const rule5 = (address) => {
    const { building_number, sub_building_name } = address;
    let premise;
    const result = utils_1.premiseLocalities(address);
    if (sub_building_name.match(STARTS_CHAR_REGEX)) {
        premise = building_number + sub_building_name;
        utils_1.prependLocality(result, premise);
    }
    else {
        premise = `${sub_building_name}, ${building_number}`;
        utils_1.prependLocality(result, building_number);
        result.push(sub_building_name);
    }
    return utils_1.combinePremise(result, address, premise, building_number, sub_building_name);
};
exports.rule5 = rule5;
/**
 * Rule 6 - Sub building name and building name
 *
 * Check the format of Sub Building Name (see Note (a) above). If the Exception
 * Rule applies, the Sub Building Name should appear on the same line as, and
 * before, the Building Name.
 *
 * Otherwise, the Sub Building Name should appear on a line preceding the Building
 * Name, Thoroughfare and Locality information
 *
 * Check format of Building Name (see note (a) above) If the Exception Rule applies,
 * the Building Name should appear at the beginning of the first Thoroughfare line,
 * or the first Locality line if there is no Thoroughfare information. Otherwise, the
 * Building Name should appear on a line preceding the Thoroughfare and Locality information.
 */
const rule6 = (address) => {
    const { sub_building_name, building_name } = address;
    let premise;
    let number = "";
    const result = utils_1.premiseLocalities(address);
    if (utils_1.nameException(sub_building_name)) {
        premise = `${utils_1.formatElem(sub_building_name)} ${building_name}`;
        result.push(premise);
    }
    else if (utils_1.nameException(building_name)) {
        premise = `${sub_building_name}, ${building_name}`;
        number = building_name;
        utils_1.prependLocality(result, building_name);
        result.push(sub_building_name);
    }
    else if (address.merge_sub_and_building) {
        premise = `${sub_building_name}, ${building_name}`;
        result.push(premise);
    }
    else {
        const sub_range_match = utils_1.checkBuildingRange(building_name);
        if (sub_range_match && !utils_1.hasUnitPrefix(building_name)) {
            // Check if name contains number range
            premise = `${sub_building_name}, ${sub_range_match.name}, ${sub_range_match.range}`;
            utils_1.prependLocality(result, sub_range_match.range);
            number = sub_range_match.range;
            result.push(sub_range_match.name);
            result.push(address.sub_building_name);
        }
        else {
            premise = `${sub_building_name}, ${building_name}`;
            result.push(building_name);
            result.push(sub_building_name);
        }
    }
    return utils_1.combinePremise(result, address, premise, number, sub_building_name);
};
exports.rule6 = rule6;
/**
 * Rule 7 - Sub building name, building name and building number
 *
 * If the Exception Rule applies, the Sub Building Name should appear on the same
 * line as and before the Building Name.
 */
const rule7 = (address) => {
    const { building_name, building_number, sub_building_name } = address;
    let result = utils_1.premiseLocalities(address);
    let premise;
    utils_1.prependLocality(result, building_number);
    if (utils_1.nameException(sub_building_name)) {
        premise = `${utils_1.formatElem(sub_building_name)} ${building_name}, ${building_number}`;
        result.push(`${utils_1.formatElem(sub_building_name)} ${building_name}`);
    }
    else if (address.merge_sub_and_building) {
        // Should not be possible to hit this code path if address object has
        // been parsed correctly
        result = utils_1.premiseLocalities(address);
        premise = `${sub_building_name}, ${building_name}`;
        result.push(premise);
    }
    else {
        premise = `${sub_building_name}, ${building_name}, ${building_number}`;
        result.push(building_name);
        result.push(sub_building_name);
    }
    return utils_1.combinePremise(result, address, premise, building_number, sub_building_name);
};
exports.rule7 = rule7;
/**
 * Undocumented Rule
 *
 * This rule should not exist as it is not listed in the developer docs. But some records
 * in the wild only have a sub building name
 */
const undocumentedRule = (address) => {
    const { sub_building_name } = address;
    const premise = sub_building_name;
    const result = utils_1.premiseLocalities(address);
    utils_1.prependLocality(result, sub_building_name);
    return utils_1.combinePremise(result, address, premise, "", sub_building_name);
};
exports.undocumentedRule = undocumentedRule;
/**
 * PO Box Rule
 */
const po_box = (address) => {
    const result = utils_1.premiseLocalities(address);
    const premise = `PO Box ${address.po_box}`;
    result.push(premise);
    return utils_1.combinePremise(result, address, premise, "", "");
};
exports.po_box = po_box;
/* tslint:disable:no-boolean-literal-compare */
const formatter = (address) => {
    if (utils_1.notEmpty(address.po_box))
        return exports.po_box(address);
    const no = utils_1.notEmpty(address.building_number); // Has building number
    const name = utils_1.notEmpty(address.building_name); // Has building name
    const sub = utils_1.notEmpty(address.sub_building_name); // Has sub building name
    if (sub === true && name === true && no === true)
        return exports.rule7(address);
    if (sub === true && name === true && no === false)
        return exports.rule6(address);
    if (sub === true && name === false && no === true)
        return exports.rule5(address);
    if (sub === true && name === false && no === false)
        return exports.undocumentedRule(address);
    if (sub === false && name === true && no === true)
        return exports.rule4(address);
    if (sub === false && name === true && no === false)
        return exports.rule3(address);
    if (sub === false && name === false && no === true)
        return exports.rule2(address);
    return exports.rule1(address); // No premise elements available
};
exports.formatter = formatter;
/* tslint:enable:no-boolean-literal-compare */
